@@ -9,18 +9,18 @@ import { DotBotsMap } from "./DotBotsMap";
 import { SailBotsMap } from "./SailBotsMap";
 import {
   apiUpdateActiveDotbotAddress, apiFetchActiveDotbotAddress,
-  apiFetchDotbots, apiUpdateRgbLed, apiUpdateMoveRaw, inactiveAddress
+  apiFetchDotbots, apiUpdateRgbLed, apiUpdateMoveRaw, apiUpdateControlMode,
+  apiUpdateWaypoints, inactiveAddress,
 } from "./rest";
+import { ApplicationType, ControlModeType, gps_distance_threshold, lh2_distance_threshold, maxWaypoints } from "./constants";
+import { gps_distance, lh2_distance } from "./helpers";
 
-const ApplicationType = {
-  DotBot: 0,
-  SailBot: 1,
-};
 
 const websocketUrl = `${process.env.REACT_APP_DOTBOTS_WS_URL}/controller/ws/status`;
 const dotbotStatuses = ["alive", "lost", "dead"];
 const dotbotBadgeStatuses = ["success", "secondary", "danger"];
 
+
 const DotBotAccordionItem = (props) => {
 
   const [ expanded, setExpanded ] = useState(false);
@@ -36,6 +36,10 @@ const DotBotAccordionItem = (props) => {
     rgbColor = `rgb(${props.dotbot.rgb_led.red}, ${props.dotbot.rgb_led.green}, ${props.dotbot.rgb_led.blue})`
   }
 
+  const autoModeClicked = async (event) => {
+    await props.updateAutoMode(props.dotbot.address, props.dotbot.application, event.target.checked);
+  }
+
   useEffect(() => {
     if (props.dotbot.rgb_led) {
       setColor({ r: props.dotbot.rgb_led.red, g: props.dotbot.rgb_led.green, b: props.dotbot.rgb_led.blue })
@@ -79,25 +83,33 @@ const DotBotAccordionItem = (props) => {
       </h2>
       <div id={`collapse-${props.dotbot.address}`} className="accordion-collapse collapse" aria-labelledby={`heading-${props.dotbot.address}`} data-bs-parent="#accordion-dotbots">
         <div className="accordion-body">
-          <div className="row">
-            <div className={`col d-flex justify-content-center ${!expanded && "invisible"}`}>
+          <div className="d-flex">
+            <div className={`mx-auto justify-content-center ${!expanded && "invisible"}`}>
               <Joystick address={props.dotbot.address} application={props.dotbot.application} />
             </div>
-            <div className="col m-2">
-              <div className="row">
-                <div className="col">
-                  <div className="d-flex justify-content-center">
-                    <RgbColorPicker color={color} onChange={setColor} />
-                  </div>
-                </div>
+            <div className="mx-auto justify-content-center">
+              <div className="d-flex justify-content-center">
+                <RgbColorPicker color={color} onChange={setColor} />
               </div>
-              <div className="col m-2">
-                <div className="d-flex justify-content-center">
-                  <button className="btn btn-primary m-1" onClick={applyColor}>Apply color</button>
-                </div>
+              <div className="d-flex justify-content-center">
+                <button className="btn btn-primary m-1" onClick={applyColor}>Apply color</button>
               </div>
             </div>
           </div>
+          {props.dotbot.lh2_position &&
+            <div className="d-flex mx-auto">
+              <div className="form-check">
+                <input className="form-check-input" type="checkbox" id="flexCheckModeAuto" defaultChecked={props.dotbot.mode} onChange={autoModeClicked} />
+                <label className="form-check-label" htmlFor="flexCheckModeAuto">Autonomous mode</label>
+              </div>
+            </div>
+          }
+          {props.dotbot.waypoints && props.dotbot.waypoints.length > 0 &&
+            <div className="d-flex mx-auto">
+              <button className="btn btn-primary btn-sm m-1" onClick={async () => props.applyWaypoints(props.dotbot.address, props.dotbot.application)}>Apply waypoints</button>
+              <button className="btn btn-primary btn-sm m-1" onClick={async () => props.clearWaypoints(props.dotbot.address, props.dotbot.application)}>Clear waypoints</button>
+            </div>
+          }
         </div>
       </div>
     </div>
@@ -128,6 +140,10 @@ const SailBotItem = (props) => {
     setActive(newSailValue !== 0);
   };
 
+  const autoModeClicked = async (event) => {
+    await props.updateAutoMode(props.dotbot.address, props.dotbot.application, event.target.checked);
+  }
+
   useInterval(async () => {
     if (rudderValue === 0 && sailValue === 0) {
       setActive(false);
@@ -184,6 +200,20 @@ const SailBotItem = (props) => {
               <button className="btn btn-primary m-1" onClick={applyColor}>Apply color</button>
             </div>
           </div>
+          {props.dotbot.gps_position &&
+            <div className="d-flex mx-auto">
+              <div className="form-check">
+                <input className="form-check-input" type="checkbox" id="flexCheckModeAuto" defaultChecked={props.dotbot.mode} onChange={autoModeClicked} />
+                <label className="form-check-label" htmlFor="flexCheckModeAuto">Autonomous mode</label>
+              </div>
+            </div>
+          }
+          {props.dotbot.waypoints && props.dotbot.waypoints.length > 0 &&
+            <div className="d-flex mx-auto">
+              <button className="btn btn-primary btn-sm m-1" onClick={async () => props.applyWaypoints(props.dotbot.address, ApplicationType.SailBot)}>Apply waypoints</button>
+              <button className="btn btn-primary btn-sm m-1" onClick={async () => props.clearWaypoints(props.dotbot.address, ApplicationType.SailBot)}>Clear waypoints</button>
+            </div>
+          }
         </div>
       </div>
     </div>
@@ -193,6 +223,8 @@ const SailBotItem = (props) => {
 const DotBots = () => {
   const [ dotbots, setDotbots ] = useState();
   const [ activeDotbot, setActiveDotbot ] = useState(inactiveAddress);
+  const [ showDotBotHistory, setShowDotBotHistory ] = useState(true);
+  const [ showSailBotHistory, setShowSailBotHistory ] = useState(true);
 
   const updateActive = useCallback(async (address) => {
     await apiUpdateActiveDotbotAddress(address).catch((error) => console.error(error));
@@ -200,14 +232,111 @@ const DotBots = () => {
   }, [setActiveDotbot]
   );
 
+  const updateAutoMode = async (address, application, mode) => {
+    const newMode = mode ? ControlModeType.Auto : ControlModeType.Manual;
+    await apiUpdateControlMode(address, application, newMode);
+    let dotbotsTmp = dotbots.slice();
+    for (let idx = 0; idx < dotbots.length; idx++) {
+      if (dotbots[idx].address === address) {
+        dotbotsTmp[idx].mode = newMode;
+        setDotbots(dotbotsTmp);
+      }
+    }
+  };
+
+  const updateShowHistory = (show, application) => {
+    if (application === ApplicationType.SailBot) {
+      setShowSailBotHistory(show);
+    } else {
+      setShowDotBotHistory(show);
+    }
+  };
+
   const fetchDotBots = useCallback(async () => {
     const data = await apiFetchDotbots().catch(error => console.log(error));
     setDotbots(data);
     const active = await apiFetchActiveDotbotAddress().catch(error => console.log(error));
     setActiveDotbot(active);
-  }, [setDotbots]
+  }, [setDotbots, setActiveDotbot]
   );
 
+  const mapClicked = useCallback((x, y) => {
+    if (!dotbots || dotbots.length === 0) {
+      return;
+    }
+
+    const activeDotbots = dotbots.filter(dotbot => activeDotbot === dotbot.address);
+    // Do nothing if no active dotbot
+    if (activeDotbots.length === 0) {
+      return;
+    }
+
+    const dotbot = activeDotbots[0];
+    if (dotbot.mode !== ControlModeType.Auto) {
+      // Do nothing if dotbot is in manual mode
+      return;
+    }
+
+    // Limit number of waypoints to maxWaypoints
+    if (dotbot.waypoints.length >= maxWaypoints) {
+      return;
+    }
+
+    if (dotbot.application === ApplicationType.SailBot) {
+      let dotbotsTmp = dotbots.slice();
+      for (let idx = 0; idx < dotbots.length; idx++) {
+        if (dotbots[idx].address === dotbot.address) {
+          if (dotbotsTmp[idx].waypoints.length === 0) {
+            dotbotsTmp[idx].waypoints.push({
+              latitude: dotbotsTmp[idx].gps_position.latitude,
+              longitude: dotbotsTmp[idx].gps_position.longitude,
+            });
+          }
+          dotbotsTmp[idx].waypoints.push({latitude: x, longitude: y});
+          setDotbots(dotbotsTmp);
+        }
+      }
+    }
+    if (dotbot.application === ApplicationType.DotBot) {
+      let dotbotsTmp = dotbots.slice();
+      for (let idx = 0; idx < dotbots.length; idx++) {
+        if (dotbots[idx].address === dotbot.address) {
+          if (dotbotsTmp[idx].waypoints.length === 0) {
+            dotbotsTmp[idx].waypoints.push({
+              x: dotbotsTmp[idx].lh2_position.x,
+              y: dotbotsTmp[idx].lh2_position.y,
+              z: 0
+            });
+          }
+          dotbotsTmp[idx].waypoints.push({x: x, y: y, z: 0});
+          setDotbots(dotbotsTmp);
+        }
+      }
+    }
+  }, [activeDotbot, dotbots, setDotbots]
+  );
+
+  const applyWaypoints = async (address, application) => {
+    for (let idx = 0; idx < dotbots.length; idx++) {
+      if (dotbots[idx].address === address) {
+        await apiUpdateWaypoints(address, application, dotbots[idx].waypoints);
+        return;
+      }
+    }
+  };
+
+  const clearWaypoints = async (address, application) => {
+    let dotbotsTmp = dotbots.slice();
+    for (let idx = 0; idx < dotbots.length; idx++) {
+      if (dotbots[idx].address === address) {
+        dotbotsTmp[idx].waypoints = [];
+        await apiUpdateWaypoints(address, application, []);
+        setDotbots(dotbotsTmp);
+        return;
+      }
+    }
+  };
+
   const onWsOpen = () => {
     console.log('websocket opened');
     fetchDotBots();
@@ -223,7 +352,11 @@ const DotBots = () => {
       let dotbotsTmp = dotbots.slice();
       for (let idx = 0; idx < dotbots.length; idx++) {
         if (dotbots[idx].address === message.address) {
-          dotbotsTmp[idx].lh2_position = {x: message.x, y: message.y};
+          const newPosition = {x: message.x, y: message.y};
+          if (dotbotsTmp[idx].lh2_position && (dotbotsTmp[idx].position_history.length === 0 || lh2_distance(dotbotsTmp[idx].lh2_position, newPosition) > lh2_distance_threshold)) {
+            dotbotsTmp[idx].position_history.push(newPosition);
+          }
+          dotbotsTmp[idx].lh2_position = newPosition;
           setDotbots(dotbotsTmp);
         }
       }
@@ -232,10 +365,23 @@ const DotBots = () => {
       let dotbotsTmp = dotbots.slice();
       for (let idx = 0; idx < dotbots.length; idx++) {
         if (dotbots[idx].address === message.address) {
-          dotbotsTmp[idx].gps_position = {
+          const newPosition = {
             latitude: message.latitude,
             longitude: message.longitude,
           };
+          if (dotbotsTmp[idx].position_history.length === 0 || gps_distance(dotbotsTmp[idx].gps_position, newPosition) > gps_distance_threshold) {
+            dotbotsTmp[idx].position_history.push(newPosition);
+          }
+          dotbotsTmp[idx].gps_position = newPosition;
+          setDotbots(dotbotsTmp);
+        }
+      }
+    }
+    if (message.cmd === "direction" && dotbots && dotbots.length > 0) {
+      let dotbotsTmp = dotbots.slice();
+      for (let idx = 0; idx < dotbots.length; idx++) {
+        if (dotbots[idx].address === message.address) {
+          dotbotsTmp[idx].direction = message.direction;
           setDotbots(dotbotsTmp);
         }
       }
@@ -284,7 +430,18 @@ const DotBots = () => {
               <div className="accordion" id="accordion-dotbots">
                 {dotbots
                   .filter(dotbot => dotbot.application === ApplicationType.DotBot)
-                  .map(dotbot => <DotBotAccordionItem key={dotbot.address} dotbot={dotbot} active={activeDotbot} updateActive={updateActive} refresh={fetchDotBots} />)
+                  .map(dotbot =>
+                    <DotBotAccordionItem
+                      key={dotbot.address}
+                      dotbot={dotbot}
+                      active={activeDotbot}
+                      updateActive={updateActive}
+                      updateAutoMode={updateAutoMode}
+                      applyWaypoints={applyWaypoints}
+                      clearWaypoints={clearWaypoints}
+                      refresh={fetchDotBots}
+                    />
+                  )
                 }
               </div>
             </div>
@@ -292,10 +449,10 @@ const DotBots = () => {
         </div>
         <div className="col col-xxl-6">
           <div className="d-block d-md-none m-1">
-            <DotBotsMap dotbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.DotBot)} active={activeDotbot} updateActive={updateActive} mapSize={350} />
+            <DotBotsMap dotbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.DotBot)} active={activeDotbot} updateActive={updateActive} showHistory={showDotBotHistory} updateShowHistory={updateShowHistory} mapClicked={mapClicked} mapSize={350} />
           </div>
           <div className="d-none d-md-block m-1">
-            <DotBotsMap dotbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.DotBot)} active={activeDotbot} updateActive={updateActive} mapSize={650} />
+            <DotBotsMap dotbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.DotBot)} active={activeDotbot} updateActive={updateActive} showHistory={showDotBotHistory} updateShowHistory={updateShowHistory} mapClicked={mapClicked} mapSize={650} />
           </div>
         </div>
       </div>
@@ -309,7 +466,18 @@ const DotBots = () => {
               <div className="accordion" id="accordion-sailbots">
                 {dotbots
                   .filter(dotbot => dotbot.application === ApplicationType.SailBot)
-                  .map(dotbot => <SailBotItem key={dotbot.address} dotbot={dotbot} active={activeDotbot} updateActive={updateActive} refresh={fetchDotBots} />)
+                  .map(dotbot =>
+                    <SailBotItem
+                      key={dotbot.address}
+                      dotbot={dotbot}
+                      active={activeDotbot}
+                      updateActive={updateActive}
+                      updateAutoMode={updateAutoMode}
+                      applyWaypoints={applyWaypoints}
+                      clearWaypoints={clearWaypoints}
+                      refresh={fetchDotBots}
+                    />
+                  )
                 }
               </div>
             </div>
@@ -317,10 +485,10 @@ const DotBots = () => {
         </div>
         <div className="col col-xxl-6">
           <div className="d-block d-md-none m-1">
-            <SailBotsMap sailbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.SailBot)} active={activeDotbot} mapSize={350} />
+            <SailBotsMap sailbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.SailBot)} active={activeDotbot} showHistory={showSailBotHistory} updateShowHistory={updateShowHistory} mapClicked={mapClicked} mapSize={350} />
           </div>
           <div className="d-none d-md-block m-1">
-            <SailBotsMap sailbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.SailBot)} active={activeDotbot} mapSize={650} />
+            <SailBotsMap sailbots={dotbots.filter(dotbot => dotbot.application === ApplicationType.SailBot)} active={activeDotbot} showHistory={showSailBotHistory} updateShowHistory={updateShowHistory} mapClicked={mapClicked} mapSize={650} />
           </div>
         </div>
       </div>
