@@ -11,7 +11,7 @@ from typing import List
 from dataclasses import dataclass
 
 
-PROTOCOL_VERSION = 2
+PROTOCOL_VERSION = 4
 
 
 class PayloadType(Enum):
@@ -23,7 +23,11 @@ class PayloadType(Enum):
     LH2_LOCATION = 3
     ADVERTISEMENT = 4
     GPS_POSITION = 5
-    INVALID_PAYLOAD = 6  # Increase each time a new payload type is added
+    DOTBOT_DATA = 6
+    CONTROL_MODE = 7
+    LH2_WAYPOINTS = 8
+    GPS_WAYPOINTS = 9
+    INVALID_PAYLOAD = 10  # Increase each time a new payload type is added
 
 
 class ApplicationType(int, Enum):
@@ -33,6 +37,13 @@ class ApplicationType(int, Enum):
     SailBot = 1  # pylint: disable=invalid-name
 
 
+class ControlModeType(int, Enum):
+    """Types of DotBot control modes."""
+
+    MANUAL = 0
+    AUTO = 1
+
+
 class ProtocolPayloadParserException(Exception):
     """Exception raised on invalid or unsupported payload."""
 
@@ -210,6 +221,32 @@ class LH2Location(ProtocolData):
         )
 
 
+@dataclass
+class DotBotData(ProtocolData):
+    """Dataclass that holds direction and LH2 raw data from DotBot application."""
+
+    direction: int = 0xFFFF
+    locations: List[Lh2RawLocation] = dataclasses.field(default_factory=lambda: [])
+
+    @property
+    def fields(self) -> List[ProtocolField]:
+        _fields = [ProtocolField(self.direction, "dir.", 2, "big", True)]
+        _fields += list(chain(*[location.fields for location in self.locations]))
+        return _fields
+
+    @staticmethod
+    def from_bytes(bytes_) -> ProtocolData:
+        return DotBotData(
+            direction=int.from_bytes(bytes_[0:2], "little", signed=True),
+            locations=[
+                Lh2RawLocation.from_bytes(bytes_[2:12]),
+                Lh2RawLocation.from_bytes(bytes_[12:22]),
+                Lh2RawLocation.from_bytes(bytes_[22:32]),
+                Lh2RawLocation.from_bytes(bytes_[32:42]),
+            ],
+        )
+
+
 @dataclass
 class GPSPosition(ProtocolData):
     """Dataclass that holds GPS positions."""
@@ -245,6 +282,59 @@ class Advertisement(ProtocolData):
         return Advertisement()
 
 
+@dataclass
+class ControlMode(ProtocolData):
+    """Dataclass that holds a control mode message."""
+
+    mode: ControlModeType = ControlModeType.MANUAL
+
+    @property
+    def fields(self) -> List[ProtocolField]:
+        return [
+            ProtocolField(self.mode, "mode"),
+        ]
+
+    @staticmethod
+    def from_bytes(bytes_) -> ProtocolData:
+        return ControlMode(bytes_[0])
+
+
+@dataclass
+class LH2Waypoints(ProtocolData):
+    """Dataclass that holds a list of LH2 waypoints."""
+
+    waypoints: List[LH2Location] = dataclasses.field(default_factory=lambda: [])
+
+    @property
+    def fields(self) -> List[ProtocolField]:
+        _fields = [ProtocolField(len(self.waypoints), "len.")]
+        _fields += list(chain(*[waypoint.fields for waypoint in self.waypoints]))
+        return _fields
+
+    @staticmethod
+    def from_bytes(_) -> ProtocolData:
+        return LH2Waypoints(waypoints=[])
+
+
+@dataclass
+class GPSWaypoints(ProtocolData):
+    """Dataclass that holds a list of GPS waypoints."""
+
+    waypoints: List[GPSPosition] = dataclasses.field(default_factory=lambda: [])
+
+    @property
+    def fields(self) -> List[ProtocolField]:
+        _fields = [ProtocolField(len(self.waypoints), "len.")]
+        _fields += list(chain(*[waypoint.fields for waypoint in self.waypoints]))
+        for field in _fields:
+            field.endian = "little"
+        return _fields
+
+    @staticmethod
+    def from_bytes(_) -> ProtocolData:
+        return GPSWaypoints(waypoints=[])
+
+
 @dataclass
 class ProtocolPayload:
     """Manage a protocol complete payload (header + type + values)."""
@@ -271,7 +361,11 @@ class ProtocolPayload:
     def from_bytes(bytes_: bytes):
         """Parse a bytearray to return a protocol payload instance."""
         header = ProtocolHeader.from_bytes(bytes_[0:20])
-        payload_type = PayloadType(int.from_bytes(bytes_[20:21], "big"))
+        if header.version != PROTOCOL_VERSION:
+            raise ProtocolPayloadParserException(
+                f"Unsupported payload version '{header.version}' (expected: {PROTOCOL_VERSION})"
+            )
+        payload_type = PayloadType(int.from_bytes(bytes_[20:21], "little"))
         if payload_type == PayloadType.CMD_MOVE_RAW:
             values = CommandMoveRaw.from_bytes(bytes_[21:26])
         elif payload_type == PayloadType.CMD_RGB_LED:
@@ -284,9 +378,17 @@ class ProtocolPayload:
             values = Advertisement.from_bytes(None)
         elif payload_type == PayloadType.GPS_POSITION:
             values = GPSPosition.from_bytes(bytes_[21:29])
+        elif payload_type == PayloadType.DOTBOT_DATA:
+            values = DotBotData.from_bytes(bytes_[21:63])
+        elif payload_type == PayloadType.CONTROL_MODE:
+            values = ControlMode.from_bytes(bytes_[21:22])
+        elif payload_type == PayloadType.LH2_WAYPOINTS:
+            values = LH2Waypoints.from_bytes(None)
+        elif payload_type == PayloadType.GPS_WAYPOINTS:
+            values = GPSWaypoints.from_bytes(None)
         else:
             raise ProtocolPayloadParserException(
-                f"Unsupported payload type {payload_type}"
+                f"Unsupported payload type '{payload_type}'"
             )
         return ProtocolPayload(header, payload_type, values)
 
@@ -310,7 +412,7 @@ class ProtocolPayload:
             for field in self.header.fields
         ]
         type_value = [
-            f" 0x{hexlify(int(PayloadType(self.payload_type).value).to_bytes(1, 'big')).decode():<3}"
+            f" 0x{hexlify(int(PayloadType(self.payload_type).value).to_bytes(1, 'little')).decode():<3}"
         ]
         values_values = [
             f" 0x{hexlify(int(field.value).to_bytes(field.length, field.endian, signed=field.signed)).decode():<{4 * field.length - 1}}"
